package com.onsiteservice.miniapp.controller.order;

import com.onsiteservice.common.annotation.user.CurrentUserId;
import com.onsiteservice.core.result.Result;
import com.onsiteservice.dao.common.page.PageInfoVO;
import com.onsiteservice.miniapp.controller.order.dto.*;
import com.onsiteservice.miniapp.controller.order.vo.ServiceOrderVO;
import com.onsiteservice.miniapp.service.order.ServiceOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.NonNull;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.Positive;

import static com.onsiteservice.core.result.ResultGenerator.fail;
import static com.onsiteservice.core.result.ResultGenerator.success;

/**
 * <P>订单模块</P>
 *
 * @author 李宁
 * @version v1.0
 * @since 2022/7/7 08:51
 */
@Api(tags = "订单模块")
@RestController
@RequestMapping("/order/")
@Validated
public class ServiceOrderController {


    @Resource
    private ServiceOrderService serviceOrderService;


    @ApiOperation(value = "根据id查询订单详情")
    @GetMapping("get/{id}")
    public Result<ServiceOrderVO> getDetail(@ApiParam(name = "id", value = "订单id") @PathVariable @Positive Long id, @CurrentUserId Long userId) {
        return success(serviceOrderService.selectById(id, userId), "根据id查询订单详情");
    }

    @ApiOperation(value = "分页查询订单")
    @PostMapping("page")
    public Result<PageInfoVO<ServiceOrderVO>> getPage(@RequestBody @NonNull @Validated PageServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.selectByUserId(dto, userId);
    }

    @ApiOperation(value = "订单预约")
    @PostMapping("reserve")
    public Result reserve(@RequestBody @NonNull @Validated ReserveServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.reserve(dto, userId) == 1 ? success() : fail("已预约");
    }

    @ApiOperation(value = "订单估价")
    @PostMapping("valuation")
    public Result valuation(@RequestBody @NonNull @Validated ValuationServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.valuation(dto, userId) == 1 ? success() : fail("估价失败");
    }

    /**
     * 发送订单 已估价订单才能发送 <link>ServiceOrderStatusEnum</link>
     */
    @ApiOperation(value = "发送订单")
    @PostMapping("send")
    public Result valuation(@RequestBody @NonNull @Validated SendServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.sendOrder(dto, userId) == 1 ? success() : fail("发送失败");
    }

    @ApiOperation(value = "支付订单")
    @PostMapping("pay")
    public Result pay(@RequestBody @NonNull @Validated PayServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.pay(dto, userId) == 1 ? success() : fail("支付失败");
    }

    @ApiOperation(value = "派单")
    @PostMapping("dispatch")
    public Result dispatch(@RequestBody @NonNull @Validated DispatchServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.dispatch(dto, userId) == 1 ? success() : fail("派单失败");
    }

    /**
     * 未派单之前不能完成,已申请退款的也可以点击完成
     */
    @ApiOperation(value = "完成订单")
    @PostMapping("finish")
    public Result finish(@RequestBody @NonNull @Validated FinishServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.finishOrder(dto, userId) == 1 ? success() : fail("取消失败");
    }

    /**
     * 订单退款 订单已付款,未派单之前可以取消 <link>ServiceOrderStatusEnum</link>
     */
    @ApiOperation(value = "订单退款")
    @PostMapping("refund")
    public Result refund(@RequestBody @NonNull @Validated RefundServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.refundOrder(dto, userId) == 1 ? success() : fail("退款失败");
    }

    /**
     * 取消订单: 未支付之前可取消订单 <link>ServiceOrderStatusEnum</link>
     */
    @ApiOperation(value = "取消订单")
    @PostMapping("cancel")
    public Result cancel(@RequestBody @NonNull @Validated CancelServiceOrderDTO dto, @CurrentUserId Long userId) {
        return serviceOrderService.cancelOrder(dto, userId) == 1 ? success() : fail("取消失败");
    }







}
