package com.onsiteservice.miniapp.service.order;

import com.github.pagehelper.PageHelper;
import com.onsiteservice.constant.constant.BizConstants;
import com.onsiteservice.constant.enums.BizCodeEnum;
import com.onsiteservice.constant.enums.ServiceOrderStatusEnum;
import com.onsiteservice.core.exception.ServiceException;
import com.onsiteservice.core.result.Result;
import com.onsiteservice.core.result.ResultGenerator;
import com.onsiteservice.dao.common.AbstractMapper;
import com.onsiteservice.dao.common.page.PageInfoVO;
import com.onsiteservice.dao.mapper.service.ServiceAddressMapper;
import com.onsiteservice.dao.mapper.service.ServiceOrderMapper;
import com.onsiteservice.dao.mapper.service.ServiceSubclassMapper;
import com.onsiteservice.dao.mapper.user.UserMapper;
import com.onsiteservice.entity.address.ServiceAddress;
import com.onsiteservice.entity.category.ServiceSubclass;
import com.onsiteservice.entity.order.ServiceOrder;
import com.onsiteservice.entity.user.User;
import com.onsiteservice.miniapp.controller.address.vo.ServiceAddressVO;
import com.onsiteservice.miniapp.controller.order.dto.*;
import com.onsiteservice.miniapp.controller.order.vo.ServiceOrderVO;
import com.onsiteservice.miniapp.mapper.order.ServiceOrderBizMapper;
import com.onsiteservice.miniapp.service.address.ServiceAddressService;
import com.onsiteservice.util.AttrCopyUtils;
import com.onsiteservice.util.RandomUtils;
import com.onsiteservice.util.aliyun.SmsUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class ServiceOrderService extends AbstractMapper<ServiceOrder> {


    @Value("${aliyun.sms.sign}")
    private String sign;

    @Autowired
    private SmsUtils smsUtils;

    @Resource
    private ServiceOrderBizMapper serviceOrderBizMapper;

    @Resource
    private ServiceAddressService serviceAddressService;

    @Resource
    private ServiceAddressMapper serviceAddressMapper;

    @Resource
    private ServiceSubclassMapper serviceSubclassMapper;

    @Resource
    private UserMapper userMapper;

    @Resource
    private ServiceOrderMapper serviceOrderMapper;


    public ServiceOrderVO selectById(Long id, Long userId) {
        // 订单信息
        Condition c = new Condition(ServiceOrder.class);
        c.createCriteria().andEqualTo("id", id).andEqualTo(BizConstants.UserConstants.ACCOUNT_NO, userId);
        List<ServiceOrder> serviceOrderList = this.selectByCondition(c);
        if (CollectionUtils.isEmpty(serviceOrderList)) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_NOT_EXIST);
        }

        ServiceOrder serviceOrder = serviceOrderList.get(0);
        ServiceOrderVO serviceOrderVO = AttrCopyUtils.copy(serviceOrder, new ServiceOrderVO());

        // 订单地址信息
        try {
            ServiceAddressVO serviceAddressVO = serviceAddressService.selectById(serviceOrder.getAddressId(), userId);

            serviceOrderVO.setAddress(serviceAddressVO.getAddress());
            serviceOrderVO.setName(serviceOrderVO.getName());
            serviceOrderVO.setPhone(serviceOrderVO.getPhone());
        } catch (Exception e) {
            log.error("根据id查询订单详情,服务地址不存在");
        }

        return serviceOrderVO;
    }


    public Result<PageInfoVO<ServiceOrderVO>> selectByUserId(PageServiceOrderDTO dto, Long uid) {
        PageHelper.startPage(dto.getPage(), dto.getSize());
        List<ServiceOrder> serviceOrders = serviceOrderBizMapper.selectServiceOrderPage(dto, uid);
        List<ServiceOrderVO> serviceOrderVOList = serviceOrders.parallelStream()
                .map(e -> AttrCopyUtils.copy(e, new ServiceOrderVO())).collect(Collectors.toList());
        return ResultGenerator.success(new PageInfoVO<>(serviceOrderVOList));
    }


    public int reserve(ReserveServiceOrderDTO dto, Long userId) {
        log.info("order reserve dto: {}, userId: {}", dto, userId);

        ServiceSubclass serviceSubclass = serviceSubclassMapper.selectByPrimaryKey(dto.getSubclassId());
        if (Objects.isNull(serviceSubclass)) {
            throw new ServiceException(BizCodeEnum.SERVICE_SUBCLASS_NOT_EXIST);
        }

        ServiceAddress serviceAddress = serviceAddressMapper.selectByPrimaryKey(dto.getAddressId());
        if (Objects.isNull(serviceAddress)) {
            throw new ServiceException(BizCodeEnum.SERVICE_ADDRESS_NOT_EXIST);
        }

        ServiceOrder serviceOrder = AttrCopyUtils.copy(dto, new ServiceOrder());
        // 账户号
        serviceOrder.setAccountNo(userId);
        // 订单号
        serviceOrder.setOrderNo(RandomUtils.orderNum());
        // 客户手机号
        serviceOrder.setPhone(serviceAddress.getPhone());

        return this.insertSelective(serviceOrder);
    }


    /**
     * 指派估价员 管理员操作
     */
    public int valuation(ValuationServiceOrderDTO dto, Long userId) {
        log.info("order valuation dto: {}, userId: {}", dto, userId);

        checkAuth(userId, BizCodeEnum.NO_AUTH_VALUATION_ORDER);

        ServiceOrder serviceOrder = checkOrder(dto.getId(), null);
        if (!serviceOrder.getOrderStatus().equals(ServiceOrderStatusEnum.SUBMIT.getStatus())) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_CANNOT_VALUATION);
        }

        serviceOrder.setOrderStatus(ServiceOrderStatusEnum.VALUATION.getStatus());

        // TODO 短信通知估价员 去估价

        // TODO 短信 "您的订单已为您指派估价员,请保持手机畅通!"

        // TODO 小程序通知客户 "您的订单已为您指派估价员,请保持手机畅通!"

        return this.updateByPrimaryKeySelective(serviceOrder);
    }


    /**
     * 发送订单 管理员操作
     */
    public int sendOrder(SendServiceOrderDTO dto, Long userId) {
        log.info("sendOrder dto: {}, userId: {}", dto, userId);

        // 判断发送者权限
        checkAuth(userId, BizCodeEnum.NO_AUTH_SEND_ORDER);

        // 判断能发送的状态
        ServiceOrder serviceOrder = checkOrder(dto.getId(), null);
        if (!serviceOrder.getOrderStatus().equals(ServiceOrderStatusEnum.VALUATION.getStatus())) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_CANNOT_SEND);
        }

        // 估价价格
        serviceOrder.setPrice(dto.getPrice());
        serviceOrder.setOrderStatus(ServiceOrderStatusEnum.SEND.getStatus());
        int result = serviceOrderMapper.updateByPrimaryKeySelective(serviceOrder);

        try {
            // 发短信
            log.info("sendOrder send msg to phone: {}", serviceOrder.getPhone());
            // TODO 短信通知客户完成支付
            smsUtils.send(sign, "", "", serviceOrder.getPhone());
        } catch (Exception e) {
            log.error("sendOrder error, ", e);
        }

        // TODO 小程序通知客户完成支付


        return result;

    }


    public int pay(PayServiceOrderDTO dto, Long userId) {
        log.info("order pay dto: {}, userId: {}", dto, userId);

        // 判断能发送的状态
        ServiceOrder serviceOrder = checkOrder(dto.getId(), userId);
        if (!serviceOrder.getOrderStatus().equals(ServiceOrderStatusEnum.SEND.getStatus())) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_CANNOT_PAY);
        }

        serviceOrder.setOrderStatus(ServiceOrderStatusEnum.PAY.getStatus());

        int result = serviceOrderMapper.updateByPrimaryKeySelective(serviceOrder);



        // TODO 调用微信支付接口

        // TODO 短信 通知管理员 -> 用户已经支付完成,请尽快指派维修工
        // TODO 小程序 通知管理员 -> 用户已经支付完成,请尽快指派维修工


        return result;
    }


    public int dispatch(DispatchServiceOrderDTO dto, Long userId) {
        log.info("dispatch dto: {}, userId: {}", dto, userId);

        checkAuth(userId, BizCodeEnum.NO_AUTH_DISPATCH_ORDER);

        ServiceOrder serviceOrder = checkOrder(dto.getId(), null);

        if (!serviceOrder.getOrderStatus().equals(ServiceOrderStatusEnum.PAY.getStatus())) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_CANNOT_DISPATCH);
        }

        serviceOrder.setOrderStatus(ServiceOrderStatusEnum.DISPATCH.getStatus());
        int result = serviceOrderMapper.updateByPrimaryKeySelective(serviceOrder);

        // TODO 短信 "您的订单已为您指派维修工,请保持手机畅通!"

        // TODO 小程序通知 您的订单已为您指派维修工~~~

        return result;
    }


    /**
     * 完成 TODO 添加定时任务已派单的 "?天" 自动完成
     */
    public int finishOrder(FinishServiceOrderDTO dto, Long userId) {
        log.info("finishOrder dto: {}, userId: {}", dto, userId);

        ServiceOrder serviceOrder = checkOrder(dto.getId(), userId);
        if (!serviceOrder.getOrderStatus().equals(ServiceOrderStatusEnum.DISPATCH.getStatus())) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_CANNOT_FINISH);
        }

        serviceOrder.setOrderStatus(ServiceOrderStatusEnum.FINISH.getStatus());
        return serviceOrderMapper.updateByPrimaryKeySelective(serviceOrder);
    }


    public int cancelOrder(CancelServiceOrderDTO dto, Long userId) {
        log.info("cancelOrder dto: {}, userId: {}", dto, userId);

        ServiceOrder serviceOrder = checkOrder(dto.getId(), userId);
        if (serviceOrder.getOrderStatus() >= ServiceOrderStatusEnum.PAY.getStatus()) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_CANNOT_CANCEL);
        }

        serviceOrder.setOrderStatus(ServiceOrderStatusEnum.CANCEL.getStatus());
        return serviceOrderMapper.updateByPrimaryKeySelective(serviceOrder);
    }


    /**
     * 未派单订单 用户可实时退款
     * 已派单订单 需要管理员审批
     */
    public int refundOrder(RefundServiceOrderDTO dto, Long userId) {
        log.info("refundOrder dto: {}, userId: {}", dto, userId);

        ServiceOrder serviceOrder = checkOrder(dto.getId(), userId);
        if (!serviceOrder.getOrderStatus().equals(ServiceOrderStatusEnum.PAY.getStatus())) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_CANNOT_REFUND);
        }

        // 已申请退款
        serviceOrder.setOrderStatus(ServiceOrderStatusEnum.REFUND_APPLY.getStatus());
        int result = serviceOrderMapper.updateByPrimaryKeySelective(serviceOrder);


        try {
            // TODO 调用微信接口退款 ??

            // TODO 通知web
        } catch (Exception e) {

        }


        return result;
    }










    private ServiceOrder checkOrder(Long id, Long userId) {
        Condition c = new Condition(ServiceOrder.class);

        c.createCriteria().andEqualTo("id", id);
        if (userId != null) {
            c.createCriteria().andEqualTo(BizConstants.UserConstants.ACCOUNT_NO, userId);
        }

        List<ServiceOrder> serviceOrderList = serviceOrderMapper.selectByCondition(c);
        if (CollectionUtils.isEmpty(serviceOrderList)) {
            throw new ServiceException(BizCodeEnum.SERVICE_ORDER_NOT_EXIST);
        }

        return serviceOrderList.get(0);
    }


    private void checkAuth(Long userId, BizCodeEnum biz) {
        User user = userMapper.selectByPrimaryKey(userId);
        if (Objects.isNull(user) || !user.getAdmin()) {
            throw new ServiceException(biz);
        }
    }


}
